@@ -8,7 +8,7 @@ from jedi.inference.base_value import ValueWrapper, ValueSet
 
 class Decoratee(ValueWrapper):
     def __init__(self, wrapped_value, original_value):
-        super(Decoratee, self).__init__(wrapped_value)
+        super().__init__(wrapped_value)
         self._original_value = original_value
 
     def py__doc__(self):
